@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -22,16 +22,26 @@
  */
 
 #include "actionmodel.h"
-#include "stdactions.h"
-#include "itemview.h"
+#include "gui/stdactions.h"
+#include "roles.h"
 
 QVariant ActionModel::data(const QModelIndex &index, int role) const
 {
     QVariant v;
     Q_UNUSED(index)
-    if (ItemView::Role_Actions==role) {
-        v.setValue<QList<Action *> >(QList<Action *>() << StdActions::self()->replacePlayQueueAction << StdActions::self()->addToPlayQueueAction);
+    switch(role) {
+    case Cantata::Role_Actions:
+        v.setValue<QList<Action *> >(QList<Action *>() << StdActions::self()->replacePlayQueueAction << StdActions::self()->appendToPlayQueueAction);
+        break;
+    case Cantata::Role_RatingCol:
+        return -1;
+    case Cantata::Role_TitleText:
+        return data(index, Qt::DisplayRole);
+    default:
+        break;
     }
     return v;
 }
 
+
+#include "moc_actionmodel.cpp"
