@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -30,11 +30,13 @@
 #include <QIcon>
 #include <QAbstractItemModel>
 #include <QList>
-#include "mpd/playlist.h"
-#include "mpd/song.h"
+#include <QMap>
+#include "mpd-interface/playlist.h"
+#include "mpd-interface/song.h"
+#include "support/icon.h"
 #include "actionmodel.h"
 
-class QMenu;
+class MirrorMenu;
 class QAction;
 
 class PlaylistsModel : public ActionModel
@@ -48,10 +50,13 @@ public:
         COL_ARTIST,
         COL_ALBUM,
         COL_YEAR,
+        COL_ORIGYEAR,
         COL_GENRE,
         COL_LENGTH,
         COL_COMPOSER,
         COL_PERFORMER,
+        COL_FILENAME,
+        COL_PATH,
 
         COL_COUNT
     };
@@ -65,9 +70,9 @@ public:
     struct PlaylistItem;
     struct SongItem : public Item, public Song
     {
-        SongItem() : parent(0) { }
-        SongItem(const Song &s, PlaylistItem *p=0) : Song(s), parent(p) { }
-        bool isPlaylist() { return false; }
+        SongItem() : parent(nullptr) { }
+        SongItem(const Song &s, PlaylistItem *p=nullptr) : Song(s), parent(p) { }
+        bool isPlaylist() override { return false; }
         PlaylistItem *parent;
     };
 
@@ -75,9 +80,8 @@ public:
     {
         PlaylistItem(quint32 k) : loaded(false), isSmartPlaylist(false), time(0), key(k) { }
         PlaylistItem(const Playlist &pl, quint32 k);
-        virtual ~PlaylistItem();
-        bool isPlaylist() { return true; }
-        void updateGenres();
+        ~PlaylistItem() override;
+        bool isPlaylist() override { return true; }
         SongItem * getSong(const Song &song, int offset);
         void clearSongs();
         quint32 totalTime();
@@ -87,7 +91,6 @@ public:
         bool loaded;
         bool isSmartPlaylist;
         QList<SongItem *> songs;
-        QSet<QString> genres;
         quint32 time;
         quint32 key;
         QDateTime lastModified;
@@ -96,38 +99,34 @@ public:
     static PlaylistsModel * self();
     static QString headerText(int col);
 
-    PlaylistsModel(QObject *parent = 0);
-    ~PlaylistsModel();
-    int rowCount(const QModelIndex &parent = QModelIndex()) const;
-    int columnCount(const QModelIndex &parent = QModelIndex()) const { Q_UNUSED(parent); return COL_COUNT; }
-    bool canFetchMore(const QModelIndex &index) const;
-    void fetchMore(const QModelIndex &index);
-    bool hasChildren(const QModelIndex &parent = QModelIndex()) const;
-    QModelIndex parent(const QModelIndex &index) const;
-    QModelIndex index(int row, int col, const QModelIndex &parent) const;
-    QVariant data(const QModelIndex &, int) const;
-    #ifndef ENABLE_UBUNTU
-    QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
-    bool setData(const QModelIndex &index, const QVariant &value, int role);
-    #endif
-    Qt::ItemFlags flags(const QModelIndex &index) const;
-    Qt::DropActions supportedDropActions() const;
+    PlaylistsModel(QObject *parent = nullptr);
+    ~PlaylistsModel() override;
+    QString name() const;
+    QString title() const;
+    QString descr() const;
+    const QIcon & icon() const { return icn; }
+    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
+    int columnCount(const QModelIndex &parent = QModelIndex()) const override { Q_UNUSED(parent) return COL_COUNT; }
+    bool canFetchMore(const QModelIndex &index) const override;
+    void fetchMore(const QModelIndex &index) override;
+    bool hasChildren(const QModelIndex &parent = QModelIndex()) const override;
+    QModelIndex parent(const QModelIndex &index) const override;
+    QModelIndex index(int row, int col, const QModelIndex &parent) const override;
+    QVariant data(const QModelIndex &, int) const override;
+    QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const override;
+    bool setHeaderData(int section, Qt::Orientation orientation, const QVariant &value, int role = Qt::EditRole) override;
+    bool setData(const QModelIndex &index, const QVariant &value, int role) override;
+    Qt::ItemFlags flags(const QModelIndex &index) const override;
+    Qt::DropActions supportedDropActions() const override;
     QStringList filenames(const QModelIndexList &indexes, bool filesOnly=false) const;
     QList<Song> songs(const QModelIndexList &indexes) const;
-    #ifndef ENABLE_UBUNTU
-    QMimeData * mimeData(const QModelIndexList &indexes) const;
-    bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int /*col*/, const QModelIndex &parent);
-    QStringList mimeTypes() const;
-    #endif
+    QMimeData * mimeData(const QModelIndexList &indexes) const override;
+    bool dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int /*col*/, const QModelIndex &parent) override;
+    QStringList mimeTypes() const override;
     void getPlaylists();
     void clear();
-    bool isEnabled() const { return enabled; }
-    void setEnabled(bool e);
-    bool exists(const QString &n) { return 0!=getPlaylist(n); }
-    #ifndef ENABLE_UBUNTU
-    QMenu * menu();
-    #endif
-    const QSet<QString> & genres() { return plGenres; }
+    bool exists(const QString &n) { return nullptr!=getPlaylist(n); }
+    MirrorMenu * menu();
     static QString strippedText(QString s);
     void setMultiColumn(bool m) { multiCol=m; }
 
@@ -141,7 +140,6 @@ Q_SIGNALS:
 
     void addToNew();
     void addToExisting(const QString &name);
-    void updateGenres(const QSet<QString> &genres);
     void updated(const QModelIndex &idx);
     void playlistRemoved(quint32 key);
 
@@ -156,25 +154,23 @@ private Q_SLOTS:
     void emitAddToExisting();
     void playlistRenamed(const QString &from, const QString &to);
     void mpdConnectionStateChanged(bool connected);
+    void coverLoaded(const Song &song, int s);
 
 private:
-    void updateGenreList();
     void updateItemMenu(bool craete=false);
     PlaylistItem * getPlaylist(const QString &name);
     void clearPlaylists();
     quint32 allocateKey();
 
 private:
-    bool enabled;
+    QIcon icn;
     bool multiCol;
     QList<PlaylistItem *> items;
     QSet<quint32> usedKeys;
-    QSet<QString> plGenres;
-    #ifndef ENABLE_UBUNTU
-    QMenu *itemMenu;
+    MirrorMenu *itemMenu;
     quint32 dropAdjust;
     QAction *newAction;
-    #endif
+    QMap<int, int> alignments;
 };
 
 #endif
