@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,8 +24,9 @@
 #ifndef DIGITALLYIMPORTED_SETTINGS_H
 #define DIGITALLYIMPORTED_SETTINGS_H
 
-#include "dialog.h"
+#include "support/dialog.h"
 #include "ui_digitallyimportedsettings.h"
+
 class LineEdit;
 class QComboBox;
 class QPushButton;
@@ -42,7 +43,7 @@ public:
 
 private Q_SLOTS:
     void login();
-    void loginStatus(bool, const QString &msg);
+    void loginStatus(bool status, const QString &msg);
 
 private:
     void setState();
